/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.portlet.tags.util;

import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.CharPool;
import com.liferay.portal.kernel.util.ParamUtil;
import com.liferay.portal.kernel.util.StringUtil;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.util.PortletKeys;
import com.liferay.portal.util.WebKeys;
import com.liferay.portlet.blogs.model.BlogsEntry;
import com.liferay.portlet.bookmarks.model.BookmarksEntry;
import com.liferay.portlet.documentlibrary.model.DLFileEntry;
import com.liferay.portlet.imagegallery.model.IGImage;
import com.liferay.portlet.journal.model.JournalArticle;
import com.liferay.portlet.messageboards.model.MBMessage;
import com.liferay.portlet.tags.NoSuchEntryException;
import com.liferay.portlet.tags.model.TagsEntry;
import com.liferay.portlet.tags.model.TagsProperty;
import com.liferay.portlet.tags.service.TagsEntryLocalServiceUtil;
import com.liferay.portlet.tags.service.TagsPropertyLocalServiceUtil;
import com.liferay.portlet.wiki.model.WikiPage;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.portlet.PortletRequest;

import javax.servlet.http.HttpServletRequest;

/**
 * <a href="TagsUtil.java.html"><b><i>View Source</i></b></a>
 *
 * @author Brian Wing Shun Chan
 *
 */
public class TagsUtil {

	public static final String[] ASSET_TYPE_CLASS_NAMES = {
		BlogsEntry.class.getName(), BookmarksEntry.class.getName(),
		DLFileEntry.class.getName(), IGImage.class.getName(),
		JournalArticle.class.getName(), MBMessage.class.getName(),
		WikiPage.class.getName()
	};

	public static final String[] ASSET_TYPE_PORTLET_IDS = {
		PortletKeys.BLOGS, PortletKeys.BOOKMARKS, PortletKeys.DOCUMENT_LIBRARY,
		PortletKeys.IMAGE_GALLERY, PortletKeys.JOURNAL,
		PortletKeys.MESSAGE_BOARDS, PortletKeys.WIKI
	};

	public static char[] INVALID_CHARACTERS = new char[] {
		CharPool.AMPERSAND, CharPool.APOSTROPHE, CharPool.AT,
		CharPool.BACK_SLASH, CharPool.CLOSE_BRACKET, CharPool.CLOSE_CURLY_BRACE,
		CharPool.COLON, CharPool.COMMA, CharPool.EQUAL, CharPool.GREATER_THAN,
		CharPool.FORWARD_SLASH, CharPool.LESS_THAN, CharPool.NEW_LINE,
		CharPool.OPEN_BRACKET, CharPool.OPEN_CURLY_BRACE, CharPool.PERCENT,
		CharPool.PIPE, CharPool.PLUS, CharPool.POUND, CharPool.QUESTION,
		CharPool.QUOTE, CharPool.RETURN, CharPool.SEMICOLON, CharPool.SLASH,
		CharPool.STAR, CharPool.TILDE
	};

	public static Set<String> addLayoutTagsEntries(
		HttpServletRequest request, List<TagsEntry> entries) {

		Set<String> layoutTagsEntries = getLayoutTagsEntries(request);

		for (TagsEntry entry : entries) {
			layoutTagsEntries.add(entry.getName());
		}

		return layoutTagsEntries;
	}

	public static Set<String> getLayoutTagsEntries(HttpServletRequest request) {
		Set<String> entries = (Set<String>)request.getAttribute(
			WebKeys.TAGS_LAYOUT_ENTRIES);

		if (entries == null) {
			entries = new HashSet<String>();

			request.setAttribute(WebKeys.TAGS_LAYOUT_ENTRIES, entries);
		}

		return entries;
	}

	public static String[] getTagsCategories(PortletRequest portletRequest) {
		return StringUtil.split(
			ParamUtil.getString(portletRequest, "tagsCategories"));
	}

	public static String[] getTagsEntries(PortletRequest portletRequest) {
		return StringUtil.split(
			ParamUtil.getString(portletRequest, "tagsEntries"));
	}

	public static boolean isValidWord(String word) {
		if (Validator.isNull(word)) {
			return false;
		}
		else {
			char[] wordCharArray = word.toCharArray();

			for (char c : wordCharArray) {
				for (char invalidChar : INVALID_CHARACTERS) {
					if (c == invalidChar) {
						if (_log.isDebugEnabled()) {
							_log.debug(
								"Word " + word + " is not valid because " + c +
									" is not allowed");
						}

						return false;
					}
				}
			}
		}

		return true;
	}

	public static String substitutePropertyVariables(
			long groupId, String entryName, String s)
		throws PortalException, SystemException {

		String result = s;

		TagsEntry entry = null;

		if (entryName != null) {
			try {
				entry = TagsEntryLocalServiceUtil.getEntry(groupId, entryName);
			}
			catch (NoSuchEntryException nsee) {
			}
		}

		if (entry != null) {
			List<TagsProperty> properties =
				TagsPropertyLocalServiceUtil.getProperties(entry.getEntryId());

			for (TagsProperty property : properties) {
				result = StringUtil.replace(
					result, "[$" + property.getKey() + "$]",
					property.getValue());
			}
		}

		return StringUtil.stripBetween(result, "[$", "$]");
	}

	public static String toWord(String text) {
		if (Validator.isNull(text)) {
			return text;
		}
		else {
			char[] textCharArray = text.toCharArray();

			for (int i = 0; i < textCharArray.length; i++) {
				char c = textCharArray[i];

				for (char invalidChar : INVALID_CHARACTERS) {
					if (c == invalidChar) {
						textCharArray[i] = CharPool.SPACE;

						break;
					}
				}
			}

			return new String(textCharArray);
		}
	}

	private static Log _log = LogFactoryUtil.getLog(TagsUtil.class);

}